import pygame
import map
import vehicle
import sign
import selectlevel
import gamestate
import mainmenu
import statescreen
import const

from pgu import gui
from pygame.locals import *
class MapCell(gui.Label):
    def __init__(self, xpos, ypos, busy=False, start=False, end=False, multicars=False, **params):
        gui.Label.__init__(self,"",**params)
        self.xpos = xpos      # x position in pixel
        self.ypos = ypos      # y position in pixel
        self.busy = busy      # high volume road
        self.start = start    # is the starting point?
        self.end = end        # is the end point?
        self.multicars = multicars  # is multiple cars allowed in the cell?
        self.vehicle = [0, False] # [current number of vehicles in the cell, is time-out]

    def getX(self):
        return self.xpos

    def getY(self):
        return self.ypos


    # returns boolean
    def isMultiCars(self):
        return self.multicars
    

    def setX(self, x):      
        self.xpos = x

    def setY(self, y):
        self.ypos = y

    def updateCell(self):
        pass
            
    # check if the cell is busy -- contains one car and doesn't allow multiple cars
    def isFree(self):
        return self.multicars or self.vehicle[0] == 0       

    # add the number of vehicles in the cell and set its status
    # return if a crash happened
    def addVehicle(self, veh):
        self.vehicle[0] += 1
        if veh.isOut():
            self.vehicle[1] = True # set is timeout
        return self.vehicle[0] > 1

    # subtract the number when vehicle moves out of the cell and set the timeout status to be false
    def removeVehicle(self):
        if self.vehicle[0] >0:
            self.vehicle[0] -= 1
            self.vehicle[1] = False

    def repaint(self):
        width = self.style.width
        height = self.style.height
        radius = height / 2 - 3

        pos = (15+ self.xpos * width + width/ 2, 15 + self.ypos * height + height / 2)
        #print pos
        if self.vehicle[0] == 1:
            if self.vehicle[1]:
                #draw timeout vehicle
                pygame.draw.circle(const.screen, const.timeOutColor, pos, radius)
            else:
                #paint vehicle
                pygame.draw.circle(const.screen, const.carColor, pos, radius)
            pygame.display.flip()
        elif self.vehicle[0] > 1:
            #paint crash
            pygame.draw.circle(const.screen, const.crashColor, pos, radius)
            pygame.display.flip()
        else:
            gui.Label.repaint(self)
        

class RoadCell(MapCell):
    def __init__(self,xpos,ypos,busy=False, start=False, end=False, multicars=False, **params):
        MapCell.__init__(self, xpos, ypos, busy, start, end, multicars, **params)

    # Update road cell
    # If it has one, update the vehicle it contains
    """
    def updateCell(self):
        print "roadCell updateCell"
        self.repaint()
        width = self.style.width
        height = self.style.height
        radius = height / 2

        pos = (15+ self.xpos * width + width/ 2, 15 + self.ypos * height + height / 2)
        if self.vehicle[0] == 1:
            #paint vehicle
            pygame.draw.circle(const.screen, (0, 0, 0), pos, radius, 0)
            pygame.display.flip()
        elif self.vehicle[0] > 1:
            #paint crash
            pygame.draw.circle(const.screen, (255, 0, 0), pos, radius, 0)
            pygame.display.flip()

"""
    # Draw itself and the vehicle it contains
    #def repaintCell(self):
    #    pass

# Intersection cell, may contain traffic sign
class IntersectionCell(MapCell):
    def __init__(self, xpos,ypos,busy=False, start=False, end=False, multicars=False, sign=None,**params):
        MapCell.__init__(self, xpos, ypos, busy, start, end, multicars, **params)
        self.sign = sign
        self.block = None
        self.direction = -1                      #0 for horizontal, 1 for vertical, -1 for unblocked

    def getSign(self):
        return self.sign

    def addStopSign(self):
        self.sign = StopSign()
        self.updateBlock()

    def setTrafficSign(self,red, green):
        self.block.updateBlock(red,green,self.direction)

    def removeSign(self):
        self.block.removeSigns()
            
    """
    def updateCell(self):
        if( not( self.vehicle == None ) ):
            if(not self.sign == None) :
                self.sign.update()
               # self.sign.handle( self.vehicle )
            #self.vehicle.update()
    """

    # Draw this cell and its content
    # Draw itself and the vehicle
    def repaint(self):
        if (self.sign) :
            self.sign.update()
            color = const.green
            if (self.sign.isRed()):
                color = const.red   
            pygame.draw.rect(const.screen,color,
                            (self.sign.signLocation[0],
                                self.sign.signLocation[1],
                                self.style.width/2,
                                self.style.height/2))
        MapCell.repaint(self)
        pygame.display.flip()
        

