import pygame
import statescreen
import componentwindow
import logwindow
import map
import cell
import vehicle
import block
import const
from statescreen import *
from pygame.locals import *
from random import *
from pgu import gui

""" The MenuDialog trigerred when the Menu button is pressed """
class MenuDialog(gui.Dialog):
    def __init__(self,gstate,**params):
        title = gui.Label("Game Menu")
        width = 400
        height = 200
        self.gstate = gstate

        # Table containing the contents (buttons) of the dialog window
        buttons = gui.Table(width=width,height=height)
        self.buttonSetup(buttons)
        gui.Dialog.__init__(self,title,buttons)

    """ Setup the buttons in the dialog window """
    def buttonSetup(self,buttons):
        resume_button = gui.Button("Back to the Game")
        restart_button = gui.Button("Restart Game")
        selectlevel_button = gui.Button("Select another level")
        mainmenu_button = gui.Button("Go to Main Menu",width=300)

        for button in [resume_button,restart_button,selectlevel_button,mainmenu_button]:
            button.connect(gui.CLICK, self.gstate.closeDialog, self)
            button.style.height = 35
            button.style.width = 300
            buttons.tr()
            buttons.td(button)

        restart_button.connect(gui.CLICK, const.toGameState, self.gstate)
        selectlevel_button.connect(gui.CLICK, const.toSelectLevel, self.gstate)
        mainmenu_button.connect(gui.CLICK, const.toMainMenu,self.gstate)

""" The ControlDialog used to control the attributes of traffic lights """
class ControlDialog(gui.Dialog):
    def __init__(self,gstate,**params):
        title = gui.Label("Traffic Sign Configuration")
        width=400
        height=200
        self.gstate = gstate
        self.rSlider = gui.HSlider(10,1,30,20,width=100)
        self.rLabel = gui.Label("", color=(255,0,0), background=(255,255,255))
        self.rSlider.connect(gui.CHANGE, self.updateLabel)
        self.gLabel = gui.Label("", color=(0,255,0), background=(255,255,255))
        self.gSlider = gui.HSlider(10,1,30,20,width=100)
        self.gSlider.connect(gui.CHANGE, self.updateLabel)

        self.rLabel.style.width = self.gLabel.style.width = 25
        self.updateLabel()
        # Table containing the content of the dialog window
        table = gui.Table()
        self.controlSetup(table)

        gui.Dialog.__init__(self,title,table)

    """ Setup the contents of the dialog window """
    def controlSetup(self, table):
        submit = gui.Button("OK",width=60, height=25)
        submit.connect(gui.CLICK,self.submitQuery)
        cancel = gui.Button("Cancel",width=60, height=25)
        cancel.connect(gui.CLICK,self.gstate.closeDialog, self)
        
        table.tr()
        table.td(gui.Label("Setup Time Interval"),height=35, colspan=2)
        table.tr()
        table.td(gui.Label("Red Interval:", color=(255,0,0)), width=150, height=25, align=-1)
        table.td(self.rSlider)
        table.tr()
        table.td(gui.Spacer(1,1))
        table.td(self.rLabel, width=20,align=-0.8)
        table.tr()
        table.td(gui.Label("Green Interval:", color=(0,255,0)), width=150, height=25, align=-1)
        table.td(self.gSlider)
        table.tr()
        table.td(gui.Spacer(1,1))
        table.td(self.gLabel,width=20,align=-0.8)
        table.tr()
        table.td(gui.Spacer(200,30), colspan=2)
        table.tr()
        table.td(submit)
        table.td(cancel)

    """ Updates the indicator labels when the slider(s) is changed """
    def updateLabel(self):
        self.rLabel.value = ('%(val)02d' % \
                            {'val':self.rSlider.value})
        self.gLabel.value = ('%(val)02d' % \
                            {'val':self.gSlider.value})
        self.rLabel.repaint()
        self.gLabel.repaint()
        
    """ Submit the user query to the gamestate. Updates the configuration
        of the sign of the clicked intersection based on the input to the
        control dialog
    """
    def submitQuery(self):
        self.gstate.logwindow.storeInfo("Configured interval:")
        self.gstate.logwindow.storeInfo("Red = "+str(self.rSlider.value)+" Green = "+str(self.gSlider.value))
        mapCells = self.gstate.map
        signLocation = None
        tempCell = self.gstate.componentwindow.cell

##        #check if intersection block at location x+1,y
##        if (mapCells.getCell(tempCell.getX()+1,tempCell.getY())) and mapCells.getCell(tempCell.getX()+1,tempCell.getY()).isMultiCars() :
##               print "is multi cars"
##               signLocation = 15+(tempCell.getX()+0.5)*tempCell.style.width, 15+(tempCell.getY()+1)*tempCell.style.height
##            #check if intersection block at location x,y+1
##        elif (mapCells.getCell(tempCell.getX(),tempCell.getY()+1)) and mapCells.getCell(tempCell.getX(),tempCell.getY()+1).isMultiCars() :
##               signLocation = 15+(tempCell.getX()-0.5)*tempCell.style.width, 15+(tempCell.getY()+0.5)*tempCell.style.height
##            #check if intersection block at location x-1,y
##        elif (mapCells.getCell(tempCell.getX()-1,tempCell.getY())) and mapCells.getCell(tempCell.getX()-1,tempCell.getY()).isMultiCars() :
##               signLocation = 15+tempCell.getX()*tempCell.style.width, 15+(tempCell.getY()-0.5)*tempCell.style.height
##            #check if intersection block at location x,y-1
##        elif (mapCells.getCell(tempCell.getX(),tempCell.getY()-1)) :
##               signLocation = 15+(tempCell.getX()+1)*tempCell.style.width, 15+tempCell.getY()*tempCell.style.height

        self.gstate.componentwindow.cell.setTrafficSign(self.rSlider.value,self.gSlider.value)
        self.gstate.componentwindow.repaint()
        self.gstate.closeDialog(self)

""" The main class of the module """
class GameState (statescreen.StateScreen):
    def __init__(self, screen, bgcolor=(0,0,0), inputfile='test.txt',**params):
        statescreen.StateScreen.__init__(self, screen, bgcolor,**params)
        self.menudialog = MenuDialog(self)
        self.controldialog = ControlDialog(self)
        self.dialogopen = False              # Indicates if a dialog window is opened
        self.logwindow = logwindow.LogWindow()
        setBorder(self.logwindow,1)
        self.componentwindow = componentwindow.ComponentWindow(self)
        setBorder(self.componentwindow,1)         
        self.gr = gui.Container(width=screen.get_width()-self.componentwindow.style.width-30,
                                height=screen.get_height()-self.logwindow.style.height-30)
        data = self.parseFile(inputfile)
        self.setTime = data[0]        # total set up time
        self.simTime = data[1]        # total simulation time
        self.timeoutTime = data[2]    # total waiting time for each vehicle before time-out
        const.timeoutAllowed = data[3] # number of time-out cars allowed
        const.timeoutCount = 0
        self.cars = data[4]           # number of vehicles that appear on the map
        self.sizex = data[5]
        self.sizey = data[6]
        self.cellwidth= data[7]
        self.cellheight = data[8]
        cells = data[9]

        # START MAKING CARS!!!
        vehicles = []
        self.map = const.map = map.Map(cells,vehicles)
        for i in range(self.cars):
            
            self.map.cars.append(vehicle.Vehicle(self.map.cells, self.timeoutTime, randint(200, 800)))

        setMin = self.setTime
        setSec = 0

        self.setuptime = gui.Label(str('%(min)02d:%(sec)02d' % {'min':self.setTime,'sec':0}),font=const.statusFont)
        self.simtime = gui.Label(str('%(min)02d:%(sec)02d' % {'min':self.simTime,'sec': 0}),font=const.statusFont)
        
        const.numTimeout = gui.Label(str('%(count)02d/%(tot)02d' % {'count':const.timeoutCount, 'tot':const.timeoutAllowed }),
                                     font=const.statusFont)
        
        self.setuptime.style.width = self.simtime.style.width = const.numTimeout.style.width = 50
        self.logwindow.storeInfo("Sim Traffic - Level 0")

        self.cont.add(self.gr,15,15)
        self.guiSetup()

    def parseFile(self, file):
        try:
            fileHandle = open('test.txt','r')

            #get setup time
            setTime = int(fileHandle.readline())

            #get the simtime
            simTime = int(fileHandle.readline())

            #get timeout time for each vehicle
            timeoutTime = int(fileHandle.readline())

            #get # of timeout cars allowed
            timeoutAllowed = int(fileHandle.readline())

            #get # of cars
            cars = int(fileHandle.readline())

            #get the size of map
            size = fileHandle.readline().split(',')
            sizex = int(size[0])
            sizey = int(size[1])

            # Dimensions of cells
            cellwidth = self.gr.style.width/(sizex)
            cellheight = self.gr.style.height/(sizey)

            cells=[]
            for i in range(sizey):
                cells.append([])
                for j in range(sizex):
                    cells[i].append(0)
            
	    #cells = [[0.0] * self.sizey] * self.sizex
            fileHandle.readline()
            for i in range(sizey):
                line = fileHandle.readline()
                print ''
                for j in range(sizex):
                    # lb is a label which is the GUI representation of each cell
                    lb = gui.Label("",background=(0,0,0))
                    #gr.add(lb,cellwidth*j,cellheight*i)        # adds cell graphic to the map
                    if line[j] == 'x' : #building
                        cells[j][i] = None
                        lb.style.background = const.noneColor
                       	print '0',
                    elif line[j] == 'i' : #intersection block
                        lb = cell.RoadCell(j,i,False, False, False, True, background = (0,0,0))
                        lb.style.background =const.blockColor
                        cells[j][i] = lb
                       	print 'i',
                    elif line[j] == 'H' : # busy (high traffic)
                        lb = cell.RoadCell(j,i,True,background = (0,0,0))
                        lb.style.background =const.busyColor
                        cells[j][i] = lb
                        print '1',
                    elif line[j] == 'L' : # not busy (low traffic)
                        lb = cell.RoadCell(j,i,background = (0,0,0))
                        lb.style.background =const.notBusyColor
                        cells[j][i] = lb
                       	print '1',
                    elif line[j] == 'S' : # start cell and busy (high traffic)
                        lb = cell.RoadCell(j,i,True,True,background = (0,0,0))
                        lb.style.background =const.busyColor
                        cells[j][i] = lb
                        print '1',
                    elif line[j] == 's' : # start cell and not busy (lwo traffic)
                        lb = cell.RoadCell(j,i,False,True,background = (0,0,0))
                        lb.style.background =const.notBusyColor
                        cells[j][i] = lb
                        print '1',
                    elif line[j] == 'E' : # end cell and busy (high traffic)
                        lb = cell.RoadCell(j,i,True, False, True,background = (0,0,0))
                        lb.style.background =const.busyColor
                        cells[j][i] = lb
                        print '1',
                    elif line[j] == 'e' : # end cell and not busy (low traffic)
                        lb = cell.RoadCell(j,i,False,False,True,background = (0,0,0))
                        lb.style.background =const.notBusyColor
                        cells[j][i] = lb
                        print '1',
                    elif line[j] == '+' : #intersection cell busy
                        lb = cell.IntersectionCell(j,i,True,background = (0,0,0))
                        lb.style.background =const.iBusyColor
                        cells[j][i] = lb
                        lb.connect(gui.CLICK,self.respond, lb)
                        print 'o',
		    elif line[j] == '#' : #intersection cell not busy
                        lb = cell.IntersectionCell(j,i,False,background = (0,0,0))
                        lb.style.background =const.iNotBusyColor
                        cells[j][i] = lb
			lb.connect(gui.CLICK,self.respond, lb)
			print 'o',
		    lb.style.width = cellwidth
                    lb.style.height = cellheight
		    self.gr.add(lb,cellwidth*j,cellheight*i)
	    return setTime,simTime,timeoutTime,timeoutAllowed,cars,sizex,sizey,cellwidth,cellheight,cells

        except IOError:
               print "The file does not exist, exiting"
        
        
    """ Override run method of gui """
    def run(self,widget=None,screen=None):
        self.init(widget,screen)
        initialTime = pygame.time.get_ticks();
        self.timer = (pygame.time.get_ticks()-initialTime)/1000;
        self.time_counter = 0
        self.set_time_minute = self.setTime
        self.set_time_second = 1
        total_setTime_secs = self.set_time_minute*60+self.set_time_second
        self.sim_time_minute = self.simTime
        self.sim_time_second = 1
        self.setTimeFinish = False
        self.simTimeFinish = False
        total_simTime_secs = self.sim_time_minute*60+self.sim_time_second

        while not self._quit:
            """ Update the maps and vehicles """
            if not self.dialogopen:
                for cellrow in self.map.cells:
                    for cell in cellrow:
                        if (cell):
                            cell.repaint()
            """ Setup time running """
            if self.timer <= total_setTime_secs and (not self.setTimeFinish):
                  self.setTimeCountdown(self.setuptime,initialTime)
            elif (not self.setTimeFinish):
                  self.setTimeFinish = True
                  initialTime = pygame.time.get_ticks();
                  self.timer = (pygame.time.get_ticks() - initialTime)/1000;
                  self.time_counter = 0

            """ Simulation time running """
            if self.setTimeFinish :
                  self.map.updateMap()
                  if self.timer <= total_simTime_secs and (not self.simTimeFinish):
                        self.simTimeCountdown(self.simtime, initialTime)
                  else :
                       self.simTimeFinish = True                       

            self.loop()
            pygame.time.wait(10)


    """ Count down for setup time """
    def setTimeCountdown(self,label,initialTime):
         if (self.timer- self.time_counter >= 1):
                 self.time_counter += 1
                 self.set_time_second -= 1
                 label.value = str('%(min)02d:%(sec)02d' % {'min':self.set_time_minute,'sec':self.set_time_second})
                 label.repaint()

                 if self.set_time_second == 0:
                    self.set_time_minute -= 1
                    self.set_time_second = 60

         self.timer = (pygame.time.get_ticks()-initialTime)/1000;

    """ Count down for simulation time """
    def simTimeCountdown(self,label,initialTime):
         if (self.timer- self.time_counter >= 1):
                self.time_counter += 1
                self.sim_time_second -= 1
                label.value = str('%(min)02d:%(sec)02d' % {'min':self.sim_time_minute,'sec':self.sim_time_second})
                label.repaint()
                if self.sim_time_second == 0:

                    self.sim_time_minute -= 1
                    self.sim_time_second = 60

         self.timer = (pygame.time.get_ticks()-initialTime)/1000;

    """ Setup the interfaces of the GameState """
    def guiSetup(self):
        #Right panel
        rpanel = gui.Table(width=self.componentwindow.style.width+20,
                           height=self.screen.get_height(),
                           background=(165,165,165))
        rpanel.tr()
        rpanel.td(self.componentwindow)
        rpanel.style.border_left=2

        #Bottom panel
        bpanel = gui.Container(width = self.screen.get_width()-rpanel.style.width,
                               height=self.logwindow.style.height+20,
                               background=(225,225,225))

        stat_table = gui.Table()
        stat_table.tr()
        stat_table.td(gui.Label("Time left before simulation starts ", font=const.labelFont),width=210, align=-1)
        stat_table.td(self.setuptime, align=-1)
        stat_table.tr()
        stat_table.td(gui.Label("Time left before simulation ends", font=const.labelFont),width=210,align=-1)
        stat_table.td(self.simtime, align=-1)
        stat_table.tr()
        stat_table.td(gui.Label("Total time out cars", font=const.labelFont),width=210, align=-1)
        stat_table.td(const.numTimeout, align=-1)

        bpanel.add(self.logwindow, bpanel.style.width-self.logwindow.style.width-10, 10)
        bpanel.add(stat_table,5,10)
        bpanel.style.border_top=2

        menuTest = gui.Button("Menu")
        menuTest.connect(gui.CLICK,self.openDialog,self.menudialog)
        
        #Adds the panels to the container
        self.cont.add(rpanel,self.screen.get_width()-rpanel.style.width,0)
        self.cont.add(bpanel,0,self.screen.get_height()-bpanel.style.height)
        self.cont.add(menuTest,0,0)

    def respond(self, cell):
        self.componentwindow.storeSignInfo(cell)
        if cell.sign == None:
            self.openDialog(self.controldialog)

    def removeSign(self):
        self.componentwindow.cell.removeSign()
        self.logwindow.storeInfo("Removed a traffic sign")
        
    def openDialog(self, dialog):
        self.dialogopen = True
        dialog.open()

    def closeDialog(self, dialog):
        self.dialogopen = False
        dialog.close()

""" Set border of specified thickness around the specified widget """
def setBorder(widget, t):
    widget.style.border_left = t
    widget.style.border_right = t
    widget.style.border_top = t
    widget.style.border_bottom = t

