import pygame
#import vehicle

# Traffic Sign Interface
#
# Defines the interface for traffic signs which have different
# behaviors in handling cars
class Sign():
    # Note: Need constructor for interface?
    def __init__(self):
        pass

    # Virtual method for handling various type of vehicle
    # Derived class will take care of this
    def handleVehicle(self, vehicle):
        pass

    # Update the traffic sign's status
    def update(self):
        pass

    # Draw the traffic sign
    def repaint(self):
        pass


# Traffic Light
# Alternate between red and green depending on the speficied time parameters
# Vehicles may proceed on green and wait on red
class TrafficLight(Sign):
    def __init__(self, redtime=None, greentime=None, signLocation = None, redLight=True):
        Sign.__init__(self)

        # Internal properties
        self.trafficClock = 0                        # traffic light clock
        self.clock = pygame.time.Clock()             # general purpose clock
        self.redTime = redtime                       # red light time
        self.greenTime = greentime                   # green light time
        self.redLight = redLight                         # true if red light is on
        self.signLocation = signLocation # signLocation[0]

        # On screen properties
        self.w = 40
        self.h = 40

        #self.trafficLightSurface = pygame.Surface.convert(pygame.image.load("trafficLight.png"))

        self.clock.tick()

    # Handle vehicles behavior
    # Vehicle stops when red is light, otherwise it may proceed
    def handleVehicle(self, vehicle):
        print "handleVehicle"
        if( self.redLight ):
            vehicle.setWaiting(True)
        else:
            vehicle.setWaiting(False)

    # Update its light status depending of the green and red timer
    def update(self):
        if(self.redLight):
            self.trafficClock += self.clock.tick()
            if( ( self.trafficClock / 1000.0 ) > self.redTime ):
                #print "CHANGE TO GREEN!!!"
                self.redLight = False
                # add: set image to green light
                self.trafficClock = 0
        else:
            self.trafficClock += self.clock.tick()
            if( ( self.trafficClock / 1000.0 ) > self.greenTime ):
                #print "CHANGE TO RED!!!"
                self.redLight = True
                # add: set image to red light
                self.trafficClock = 0

    # Red light time setter
    def setRedTime(self, redTime):
        self.redTime = redTime

    # Green light time setter
    def setGreenTime(self, greenTime):
        self.greenTime = greenTime

    # Red light time getter
    def getRedTime(self):
        return self.redtime

    # Green light time getter
    def getGreenTime(self):
        return self.greentime

    # Returns true if red light is on
    def isRed(self):
        return self.redLight

    # Draw itself
    def repaint(self):
        #pygame.draw.rect(self.trafficLightSurface, (200,200,200), pygame.Rect(self.x, self.y, self.w, self.h))
        pass


"""
# Stop Sign
#
# Represents a stop sign which handle vehicles such that
# all vehicles in the particular intersection can take turn
# in proceeding
class StopSign(Sign):
    def __init__(self):
        Sign.__init__(self)

        #self.clock = pygame.time.Clock()
        #self.haveCar = False

    def handleVehicle(self,vehicle):
        #self.haveCar = True
        #vehicle.setStop(2)
        pass

    def update(self):
        pass

    def repaint(self):
        pass
        

def main():
    screen = pygame.display.set_mode((800,600))
    pygame.display.set_caption(":: Sim Traffic ::")
    test = TrafficLight(100,100,4,2)
    test.setRedTime(5)
    test.setGreenTime(3)

    #testVec = vehicle.Vehicle()

    while(True):
        test.update()
        #test.handleVehicle(testVec)
        test.repaint()
        pygame.display.update()

main()
"""
