import pygame
import cell
import types
import const
import map
import gamestate
from pygame.locals import *
from random import *
from pgu import gui

class Vehicle:
    def __init__(self, cells, timeoutTime, speed=500):
        self.time = timeoutTime             # get the timeout time from gamestate class
        self.xpos = 0                       # Vehicle's x position on the screen
        self.ypos = 0                       # Vehicle's y position on the screen
        self.timeoutTime = 0                # keep track how long the vehicle has waited for stop sign or red traffic light
        self.timeout = False                # Determines whether vehicle is timeouted or not
        self.path = []                      # List of mapcells
        self.passedpath = -1                # Index/Iterator through the path array, determines where the vehicle is at the moment
        self.iswaiting = False              # Is the vehicle waiting at a stop sign or for red traffic light?
        self.isActive = True
        #self.color
        self.speed = speed                  # How many seconds the vehicle takes to move to the next level
        self.clock = pygame.time.Clock()    # General purpose clock to count time
        self.clockTimeOut = pygame.time.Clock()    # Clock for keeping track how long the vehicle has waited for stop sign or red traffic light
        self.vehTimer = 0                   # How long the clock
        self.clock.tick()
        self.clockTimeOut.tick()
        self.computePath(cells)
	self.firstWaiting = False	    # this is a helper variable for increaseTime function

    # Return true if the vehicle is timeouted (waited too long)
    def isOut(self):
        return self.timeout
    
    # Return true is the vehicle is still waiting at stop sign or for red traffic light
    def isWaiting(self):
        return self.iswaiting
    
    # Set the vehicle's waiting status
    def setWaiting( self, waiting ):
        self.iswaiting = waiting
	self.increaseTime()

    # Set the vehicle's timeout status
    def increaseTime(self):
        if self.isWaiting():
	    if not self.firstWaiting:
		self.clockTimeOut.tick()
		self.firstWaiting = True
	    else:	
		self.timeoutTime += self.clockTimeOut.tick()
	else:
	    if self.firstWaiting:
		self.timeoutTime += self.clockTimeOut.tick()
		self.firstWaiting = False
	    
        if self.timeoutTime > self.time:
	    if not self.isOut():
		self.timeout = True
		print "waiting timeout"
		self.addNumTimeout()

    # set the vehicle to timeout if crashed
    def setCrashed(self):
        self.timeout = True
        self.addNumTimeout()

    # increment the count label on the status bar
    def addNumTimeout(self):
        const.timeoutCount += 1
        const.numTimeout.value = str('%(count)02d/%(tot)02d' % {'count':const.timeoutCount, 'tot':const.timeoutAllowed })
        const.numTimeout.repaint()

    


    # Update this vehicle status
    # The car will try to move to its next cell (based on its precomputed path) if:
    # 1. Its timer has passed its speed timer and
    # 2. The vehicle is not waiting at a stop sign or red traffic light
    def updateVehicle(self):
        #if not isinstance(self.path[self.passedpath], cell.RoadCell):
        #    print "isinstance"
        #    self.path[self.passedpath].getSign().handleVehicle(self)
        #elif isinstance(self.path[self.passedpath], cell.RoadCell):
        #    print "roadCell"
        if(self.vehTimer > self.speed): #and not(self.isWaiting)):
            self.move()
            self.vehTimer = 0
  
        self.vehTimer += self.clock.tick()

    # Move the car to the next cell after checking the status of the next cell
    # emptying the cell it's currently in
    # and deleting the car if it's at the end of its path
    def move(self):
        if self.passedpath+1 == len(self.path):
            print "REACHED"
            self.path[self.passedpath].removeVehicle()
            self.isActive = False
        else:
            # TRAFFIC LIGHT CHECK HERE
                # if it's intersection cell
            if isinstance(self.path[self.passedpath], cell.IntersectionCell):
                sign = self.path[self.passedpath].getSign()
            
                if sign and sign.isRed():
                    self.setWaiting(True)
                else:
                     self.moveHelper()
                # if road cell but next cell is not free      
            elif not self.path[self.passedpath+1].isFree():
                self.setWaiting(True)
                #
            else:
                self.moveHelper()


    def moveHelper(self):
        if not self.waitTurn():
            self.setWaiting(False)
            self.path[self.passedpath].removeVehicle()
            self.passedpath += 1
            if self.path[self.passedpath].addVehicle(self): # if the car crashed
                if not self.isOut():
                    self.setCrashed()
        else:
            self.setWaiting(True)


    # this will prevent crashes when the traffic signs are correctly set
    # i.e. traffic from opposite directions will yield when turning
    def waitTurn(self):
        
        if self.passedpath > 0 and self.passedpath < len(self.path)-1:
            if self.path[self.passedpath-1].getY() == self.path[self.passedpath].getY():
                if self.path[self.passedpath].getY() < self.path[self.passedpath+1].getY():
                    if const.map.getCell(self.path[self.passedpath+1].getX()-1, self.path[self.passedpath+1].getY()) and not const.map.getCell(self.path[self.passedpath+1].getX()-1, self.path[self.passedpath+1].getY()).vehicle[0]==0 and not const.map.getCell(self.path[self.passedpath+1].getX()-1, self.path[self.passedpath+1].getY()).getSign().isRed():
                        return True
                elif self.path[self.passedpath].getY() > self.path[self.passedpath+1].getY():
                    if const.map.getCell(self.path[self.passedpath+1].getX()+1, self.path[self.passedpath+1].getY()) and not const.map.getCell(self.path[self.passedpath+1].getX()+1, self.path[self.passedpath+1].getY()).vehicle[0]==0 and not const.map.getCell(self.path[self.passedpath+1].getX()+1, self.path[self.passedpath+1].getY()).getSign().isRed():
                        return True

                return False
            elif self.path[self.passedpath-1].getX() == self.path[self.passedpath].getX():
                if self.path[self.passedpath].getX() > self.path[self.passedpath+1].getX():
                    if const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()-1) and not const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()-1).vehicle[0]==0 and not const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()-1).getSign().isRed():
                        return True
                elif self.path[self.passedpath].getX() < self.path[self.passedpath+1].getX():
                    if const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()+1) and not const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()+1).vehicle[0]==0 and not const.map.getCell(self.path[self.passedpath+1].getX(), self.path[self.passedpath+1].getY()+1).getSign().isRed():
                        return True

                return False


            else:
                print "Error with the path: going in diagonal direction"
                    
                                                                                              


    
    
    # Precompute its path for the specified map
    def computePath(self, cells):	# cells = map.cells
        incYpos = decYpos = incXpos = decXpos = True	# default: vehicle can go to any direction
        right = left = straight = False			# default: Low-traffic
        nextHigh = []   # cells with high-traffic
        nextLow = []    # cells with low-traffic
        startHigh = []	# start cells with high traffic
        startLow = []	# start cells with low traffic
        nextPath = ""
        indPrint = 0	# index for testing, printing
        rand = Random()
        secondCell = False
        
        x = y = 0
        while x < len(cells):
            while y < len(cells[0]):
                if not (type(cells[x][y]) == type(None)) and cells[x][y].start:
                    if cells[x][y].busy:
                        startHigh.append((x, y))
                        # for testing: print "start", x, y
                    else:
                        startLow.append((x, y))
                        # for testing: print "start", x, y
                y += 1
            x += 1
            y = 0
        # get random coordinate
        print ""
        if rand.randint(0, 4) > 0:
            random = rand.randint(0, len(startHigh)- 1)
            xpos = startHigh[random][0]
            ypos = startHigh[random][1]
            self.path.append(cells[xpos][ypos])
            print xpos, ypos
        else:
            random = rand.randint(0, len(startLow)- 1)
            xpos = startLow[random][0]
            ypos = startLow[random][1]
            self.path.append(cells[xpos][ypos])
            print xpos, ypos
        
        while True: # compute path
            if incXpos:	# when a vehicle goes on incremented x-axis
                if ((xpos+1) < len(cells)) and not (type(cells[xpos+1][ypos]) == type(None)) and isinstance(cells[xpos+1][ypos], cell.RoadCell):	# at a road cell    
                    if not cells[xpos+1][ypos].end:
                        self.path.append(cells[xpos+1][ypos])
                        xpos += 1
                        print xpos, ypos
                        secondCell = True
                        incXpos = True
                        decXpos = False
                        incYpos = False
                        decYpos = False
                    elif cells[xpos+1][ypos].end:  # if end point
                        if secondCell:
                            self.path.append(cells[xpos+1][ypos])
                            xpos += 1
                            print xpos, ypos
	                    break
                elif xpos + 1 < len(cells) and not (type(cells[xpos+1][ypos]) == type(None)) and isinstance(cells[xpos+1][ypos], cell.IntersectionCell):  # at an intersection cell
                    xpos += 1
                    self.path.append(cells[xpos][ypos])
                    print xpos, ypos
                    secondCell = True
                    # to get the traffic condition of going straight, left, or right
                    if not (type(cells[xpos+1][ypos+1]) == type(None)) and cells[xpos+1][ypos+1].busy:
                        nextHigh.append("right")
                    else:
                        nextLow.append("right")
                    if not (type(cells[xpos+2][ypos-2]) == type(None)) and cells[xpos+2][ypos-2].busy:
                        nextHigh.append("left")
                    else:
                        nextLow.append("left")
                    if not (type(cells[xpos+3][ypos]) == type(None)) and cells[xpos+3][ypos].busy:
                        nextHigh.append("straight")
                    else:
                        nextLow.append("straight")
                    if len(nextLow) == 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    elif len(nextHigh) == 0:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    elif rand.randint(0, 4) > 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    else:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    ####CHOOSE WHICH WAY###
                    
                    
                    if nextPath == "right":
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        incXpos = False
                        incYpos = True
                    elif nextPath == "left":
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        incXpos = False
                        decYpos = True
                    elif nextPath == "straight":
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
            if decXpos:	# when a vehicle goes on decremented x-axis
                if (xpos - 1 >= 0) and not (type(cells[xpos-1][ypos]) == type(None)) and isinstance(cells[xpos-1][ypos], cell.RoadCell):	# at a road cell
                    if not cells[xpos-1][ypos].end:
                        self.path.append(cells[xpos-1][ypos])
                        xpos -= 1
                        print xpos, ypos
                        secondCell = True
                        incXpos = False
                        decXpos = True
                        incYpos = False
                        decYpos = False
                    elif cells[xpos-1][ypos].end:  # if end point
                        if secondCell:
                            self.path.append(cells[xpos-1][ypos])
                            xpos -= 1
                            print xpos, ypos
                            break
                elif xpos - 1 >= 0 and not (type(cells[xpos-1][ypos]) == type(None)) and isinstance(cells[xpos-1][ypos], cell.IntersectionCell):	# at an intersection cell
                    xpos -= 1
                    self.path.append(cells[xpos][ypos])
                    print xpos, ypos
                    secondCell = True
		    # to get the traffic condition of going straight, left, or right
		    if not (type(cells[xpos-1][ypos-1]) == type(None)) and cells[xpos-1][ypos-1].busy:
		        nextHigh.append("right")
		    else:
		        nextLow.append("right")
		    if not (type(cells[xpos-2][ypos+2]) == type(None)) and cells[xpos-2][ypos+2].busy:
		        nextHigh.append("left")
		    else:
		        nextLow.append("left")
		    if not (type(cells[xpos-3][ypos]) == type(None)) and cells[xpos-3][ypos].busy:
		        nextHigh.append("straight")
		    else:
                        nextLow.append("straight")
                    if len(nextLow) == 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    elif len(nextHigh) == 0:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    elif rand.randint(0, 4) > 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    else:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    ####CHOOSE WHICH WAY###
                    
                    if nextPath == "right":
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        decXpos = False
                        decYpos = True
                    elif nextPath == "left":
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        decXpos = False
                        incYpos = True
                    elif nextPath == "straight":
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
            if incYpos:	# when a vehicle goes on incremented y-axis
                if (ypos + 1 < len(cells[0])) and not (type(cells[xpos][ypos+1]) == type(None)) and isinstance(cells[xpos][ypos+1], cell.RoadCell):	# at a road cell     
                    if not cells[xpos][ypos+1].end:
                        self.path.append(cells[xpos][ypos+1])
                        ypos += 1
                        print xpos, ypos
                        secondCell = True
                        incXpos = False
                        decXpos = False
                        incYpos = True
                        decYpos = False
                    elif cells[xpos][ypos+1].end:  # if end point
                        if secondCell:
	                    self.path.append(cells[xpos][ypos+1])
	                    ypos += 1
	                    print xpos, ypos
                            break
                elif ypos + 1 < len(cells[0]) and not (type(cells[xpos][ypos+1]) == type(None)) and isinstance(cells[xpos][ypos+1], cell.IntersectionCell):	# at an intersection cell
                    ypos += 1
                    self.path.append(cells[xpos][ypos])
                    print xpos, ypos
                    secondCell = True
		    # to get the traffic condition of going straight, left, or right
		    if not (type(cells[xpos-1][ypos+1]) == type(None)) and cells[xpos-1][ypos+1].busy:
			nextHigh.append("right")
		    else:
			nextLow.append("right")
		    if not (type(cells[xpos+2][ypos+2]) == type(None)) and cells[xpos+2][ypos+2].busy:
			nextHigh.append("left")
		    else:
			nextLow.append("left")
		    if not (type(cells[xpos][ypos+3]) == type(None)) and cells[xpos][ypos+3].busy:
			nextHigh.append("straight")
		    else:
                        nextLow.append("straight")
                    if len(nextLow) == 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    elif len(nextHigh) == 0:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    elif rand.randint(0, 4) > 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    else:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    ####CHOOSE WHICH WAY###
                    
                    if nextPath == "right":
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        incYpos = False
                        decXpos = True
                    elif nextPath == "left":
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        incYpos = False
                        incXpos = True
                    elif nextPath == "straight":
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
            if decYpos:	# when a vehicle goes on decremented y-axis
                if (ypos - 1 >= 0) and not (type(cells[xpos][ypos-1]) == type(None)) and isinstance(cells[xpos][ypos-1], cell.RoadCell):	# at a road cell     
                    if not cells[xpos][ypos-1].end:
	                self.path.append(cells[xpos][ypos-1])
	                ypos -= 1
	                print xpos, ypos
	                secondCell = True
                        incXpos = False
                        decXpos = False
                        incYpos = False
                        decYpos = True
                    elif cells[xpos][ypos-1].end:  # if end point
                        if secondCell:
                                self.path.append(cells[xpos][ypos-1])
                                ypos -= 1
                                print xpos, ypos
                                break
                elif ypos - 1 >= 0 and not (type(cells[xpos][ypos-1]) == type(None)) and isinstance(cells[xpos][ypos-1], cell.IntersectionCell):	# at an intersection cell
                    ypos -= 1
                    self.path.append(cells[xpos][ypos])
                    print xpos, ypos
                    secondCell = True
                    # to get the traffic condition of going straight, left, or right
		    if not (type(cells[xpos+1][ypos-1]) == type(None)) and cells[xpos+1][ypos-1].busy:
		    	nextHigh.append("right")
		    else:
		    	nextLow.append("right")
		    if not (type(cells[xpos-2][ypos-2]) == type(None)) and cells[xpos-2][ypos-2].busy:
		    	nextHigh.append("left")
		    else:
		    	nextLow.append("left")
		    if not (type(cells[xpos][ypos-3]) == type(None)) and cells[xpos][ypos-3].busy:
		    	nextHigh.append("straight")
		    else:
                        nextLow.append("straight")
                    if len(nextLow) == 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    elif len(nextHigh) == 0:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    elif rand.randint(0, 4) > 0:
                        nextPath = nextHigh[rand.randint(0, len(nextHigh) - 1)]
                    else:
                        nextPath = nextLow[rand.randint(0, len(nextLow) - 1)]
                    ####CHOOSE WHICH WAY###
                    if nextPath == "right":
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos += 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        decYpos = False
                        incXpos = True
                    elif nextPath == "left":
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        xpos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        decYpos = False
                        decXpos = True
                    elif nextPath == "straight":
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
                        ypos -= 1
                        self.path.append(cells[xpos][ypos])
                        print xpos, ypos
            # remove the content of nextHigh and nextLow
            for x in nextHigh:
                nextHigh.remove(x)
            for x in nextLow:
                nextLow.remove(x)
            
        
class Car(Vehicle):
    def __init__(self, cells, timeoutTime):	# mp = map, timeoutTime = gamestate.timeoutTime
        Vehicle.__init__(self, cells, timeoutTime)
        Vehicle.computePath(self, cells)
        self.color = None     # Decide this later


