import sys
sys.path.append('../src')
import pygame
import gamestate
import unittest
import vehicle


class TestVehicleFunctions(unittest.TestCase):

    def setUp(self):
        self.vehicle = vehicle.Vehicle(100, 500) #pass random value for timeouttime and speed
        try:
            fileHandle = open('test.txt','r')
            
            for blank in range(5):
                fileHandle.readline()

            size = fileHandle.readline().split(',')
            sizex = int(size[0])
            sizey = int(size[1])
            fileHandle.readline()

            self.cells=[]
            
            for i in range(sizey):
                self.cells.append([])
                for j in range(sizex):
                    self.cells[i].append(0)
            
            for i in range(sizey):
                line = fileHandle.readline()
                
                for j in range(sizex):
                    self.cells[j][i] = line[j]
        except IOError:
            print "The file does not exist, exiting"

        

    def testcomputepath(self):
        self.vehicle.path = self.vehicle.computePath(self.cells)

        

        
        # vihecles never get out of the road (roadcell or intersectioncell)
        for p in self.vehicle.path:
            x = (p.getX - 15) / p.style.width
            y = (p.getY - 15) / p.style.height
            self.assertTrue(not (cells[x][y] == 'x',
                                 'vehicle path not on the road'))

        
        # the first and the last on the path is on the edge of the map 
        startCell = self.vehicle.path[0]
        endCell = self.vehicle.path[len(self.vehicle.path)-1]
        x = (startCell.getX - 15) / startCell.style.width
        y = (startCell.getY - 15) / startCell.style.height

        self.assertTrue(((cells[x][y] == 's') or (cells[x][y] == 'S')),
                        'Path doesn\'t start on the edge of the map')
                   
        x = (endCell.getX - 15) / endCell.style.width
        y = (endCell.getY - 15) / endCell.style.height
            
        self.assertTrue(((cells[x][y] == 'e') or (cells[x][y] == 'E')),
                        'Path doesn\'t end on the edge of the map')


        # vehicles only move to the neighboring cell            
        pBefore = self.vehicle.path[0]
        for i in range(1, len(self.vehicle.path)):
            pAfter = self.vehicle.path[i]

            x1 = (pBefore.getX - 15) / endCell.style.width
            y1 = (pBefore.getY - 15) / endCell.style.height 
            
            x2 = (pAfter.getX - 15) / endCell.style.width
            y2 = (pAfter.getY - 15) / endCell.style.height    

            xDiff = abs(x1 - x2)
            yDiff = abs(y1 - y2)
            
            self.assertTrue((((xDiff == 1) and (yDiff == 0)) or ((yDiff == 1) and (xDiff == 0))), 'Vehicles don\'t move to the neighboring cell')
            pBefore = pAfter
        
        # vehicles always move forward
        

    def testmove(self):
        
        pass
    
    def testupdate(self):
        pass





if __name__ == '__main__':
    
    suite = unittest.TestLoader().loadTestsFromTestCase(TestVehicleFunctions)

    unittest.TextTestRunner(verbosity=2).run(suite)

